
# Many parts are ripped out of Mojo::Template by Sebastian Riedel.

package Cobalt::Template;

use base 'Cobalt::Base';

use Carp 'croak';
use warnings;
use strict;

__PACKAGE__->attr([qw/compiled namespace/]);

sub new {
	my $class = shift;
	my %new = @_;
	my $self = {
		'file' => undef,
		'vars' => undef,
		'tree' => [],
		'code' => undef,
		'directive' => undef,
	};
	return bless $self, $class;
}


sub render {
	my $self = shift;
	$self->{file} = shift;
	$self->{vars} = $_[0];
	
	$self->parse();
	$self->build();
	my $e = $self->compile;
	return $e if $e;
	return $self->interpret(@_);
	
}

sub interpret () {
	my $self = shift;

	# Compile
	unless ($self->compiled) {
		my $e = $self->compile;
		
		# Exception
		return $e if ref $e;
	}
	my $compiled = $self->compiled;

	# Shortcut
	return unless $compiled;

	# Catch warnings
	#local $SIG{__WARN__} = sub { warn Mojo::Template::Exception->new(shift, $self->template) };

	# Catch errors
	#local $SIG{__DIE__} = sub { die Mojo::Template::Exception->new(shift, $self->template) };

	# Interpret
	my $output = eval { $compiled->(@_) };
	$output = $@ if $@;
	
	return $output;
}

sub compile {
	my $self = shift;

	# Shortcut
	my $code = $self->{code};
	return unless $code;

	# Compile
	my $compiled = eval $code;

	# Exception
	#return Mojo::Template::Exception->new($@, $self->template) if $@;
	return 0 if $@;

	$self->compiled($compiled);
	return;
}

sub build {
	my $self = shift;
	
	my @lines;
	
	# Build internal vars
	for my $i (keys %{$self->{vars}}) {
		if (ref $self->{vars}->{$i} eq 'ARRAY') {
			push @lines, "my \$$i = \$self->{vars}->{$i};";
		} elsif (ref $self->{vars}->{$i} eq 'HASH') {
			push @lines, "my \$$i = \$self->{vars}->{$i};";
		} elsif (ref $self->{vars}->{$i} eq 'CODE') {
			push @lines, "\$self->{vars}->{$i}->();";		# TODO: CEK ERROR !
		} else {
			push @lines, "my \$$i = '$self->{vars}->{$i}';";
		}
		#print "my \$$i = \$self->{vars}->{$i};\n";
	}
	
	# Compile
	for my $line (@{$self->{tree}}) {
		# New line
		push @lines, '';
		for (my $j = 0; $j < @{$line}; $j += 2) {
			my $type  = $line->[$j];
			my $value = $line->[$j + 1];
			
			# Need to fix line ending?
			my $newline = chomp $value;
			
			# Text
			if ($type eq 'text') {
			
			# Quote and fix line ending
			$value = quotemeta($value);
			$value .= '\n' if $newline;
			
			$lines[-1] .= "\$_M .= \"" . $value . "\";";
			}
			
			# Code
			if ($type eq 'code') {
				$lines[-1] .= "$value";
			}
			
			# Expression
			if ($type eq 'expr') {
				$lines[-1] .= "\$_M .= $value;";
				print "-> $value\n";
			}
			
			# Expression that needs to be escaped
			if ($type eq 'escp') {
				#$lines[-1] .= "\$_M .= Mojo::ByteStream->new($value)->xml_escape;";
			}
		}
	}
	
	# Wrap
	my $namespace = $self->namespace || ref $self;
	$lines[0] ||= '';
	$lines[0] = qq/package $namespace; sub { my \$_M = '';/ . $lines[0];
	$lines[-1] .= q/return $_M; };/;

	$self->{code} = join "\n", @lines;
	return $self;

}

sub parse {
	my $self = shift;
	my %vars = %{$self->{vars}};	

	my $line_start = quotemeta '%';
	my $tag_start  = quotemeta '<%';
	my $tag_end= quotemeta '%>';
	my $cmnt_mark  = quotemeta '#';		# Perl comment style
	my $cmnt2_mark  = quotemeta '--';		# JSP comment style
	my $escp_mark  = quotemeta '=';
	my $expr_mark  = quotemeta '=';	# Expression
	my $direct_mark  = quotemeta '@';	# Directive eg: <%@ include file="header.html" %>
	
	open FH, '<' . $self->{file};
	my $state = 'text';
	my $multiline_expression = 0;
	while (my $line = <FH>) {
		# Perl line without return value
		if ($line =~ /^$line_start\s+(.+)$/) {
			push @{$self->{tree}}, ['code', $1];
			$multiline_expression = 0;
			next;
		}
		
		# Perl line with return value that needs to be escaped
		if ($line =~ /^$line_start$expr_mark$escp_mark\s+(.+)$/) {
			push @{$self->{tree}}, ['escp', $1];
			$multiline_expression = 0;
			next;
		}
		
		# Perl line with return value
		if ($line =~ /^$line_start$expr_mark\s+(.+)$/) {
			push @{$self->{tree}}, ['expr', $1];
			$multiline_expression = 0;
			next;
		}
		
		# Comment line, dummy token needed for line count
		if ($line =~ /^$line_start$cmnt_mark\s+(.+)$/) {
			push @{$self->{tree}}, [];
			$multiline_expression = 0;
			next;
		}
		
		###################
		# Mixed line
		my @token;
		for my $token (
		split /
		(
		$tag_start$expr_mark$escp_mark   # Escaped expression
		|
		$tag_start$expr_mark # Expression
		|
		$tag_start$cmnt_mark # Comment
		|
		$tag_start   # Code
		|
		$tag_end # End
		)
		/x, $line
		  )
		{
		
			# Garbage
			next unless $token;
			
			# End
			if ($token =~ /^$tag_end$/) {
			$state = 'text';
			$multiline_expression = 0;
			}
			
			# Code
			elsif ($token =~ /^$tag_start$/) { $state = 'code' }
			
			# Comment
			elsif ($token =~ /^$tag_start$cmnt_mark$/) { $state = 'cmnt' }
			
			# Expression
			elsif ($token =~ /^$tag_start$expr_mark$/) { $state = 'expr'; }
			
			# Expression that needs to be escaped
			elsif ($token =~ /^$tag_start$expr_mark$escp_mark$/) { $state = 'escp'; }
			
			# Value
			else {
			
				# Comments are ignored
				next if $state eq 'cmnt';
				
				# Multiline expressions are a bit complicated,
				# only the first line can be compiled as 'expr'
				$state = 'code' if $multiline_expression;
				$multiline_expression = 1 if $state eq 'expr';
				
				# Store value
				push @token, $state, $token;
			}
		}
			  
		push @{$self->{tree}}, \@token;
	
	}
	
	close FH;
	
	#print $self->{tree}->[8][1];
	
	return $self;
}

sub directive_page {
	my $self = shift;
	# <%@ page contentType="text/html; charset=iso-8859-1" language="java" import="java.sql.*" errorPage="" %>
}

sub directive_include {
	my $self = shift;
	# <%@ include file="header.html" %>
}

1;











